/*
	
	ArpLView.cpp
	
	Copyright (c)1998 by Angry Red Planet.

	This code is distributed under a modified form of the
	Artistic License.  A copy of this license should have
	been included with it; if this wasn't the case, the
	entire package can be obtained at
	<URL:http://www.angryredplanet.com/>.

	A basic BView that is also an ArpLayoutable.
*/

#ifndef ARPLAYOUT_ARPVIEW_H
#include "ArpLayout/ArpLView.h"
#endif

#ifndef ARPKERNEL_ARPDEBUG_H
#include "ArpKernel/ArpDebug.h"
#endif

#include <be/support/Autolock.h>
#include <float.h>

ArpMOD();

/* ------ ArpLView constructor and destructor ------
 *
 * The various ways to create and destroy ArpLView
 * objects.
 */
 
ArpLView::ArpLView(const char* name, uint32 flags)
	:BView(BRect(0,0,1,1),name,B_FOLLOW_NONE,flags),
	 ArpViewWrapper(this)
{
	OwnView();
	SetLayoutName(name);
}

ArpLView::ArpLView(BRect frame, const char* name, uint32 resizeMask, uint32 flags)
	:BView(frame,name,resizeMask,flags),
	 ArpViewWrapper(this)
{
	OwnView();
	SetLayoutName(name);
}

ArpLView::ArpLView(BMessage* data)
	: BView(data), ArpViewWrapper(data, this)
{
	OwnView();
}

/* ------------ ArpLView archiving ------------
 *
 * Archiving and retrieving ArpLView objects.
 */
BArchivable* ArpLView::Instantiate(BMessage* archive)
{
	if ( validate_instantiation(archive, "ArpLView") ) 
		return new ArpLView(archive); 
	return NULL;
}

status_t ArpLView::Archive(BMessage* data, bool deep)
{
	status_t status = BView::Archive(data,false);
	if( status == B_NO_ERROR )
		status = ArpLayoutable::Archive(data,deep);
	if( status == B_NO_ERROR )
		status = data->AddString("class","ArpLView");
	return status;
}

void ArpLView::Draw(BRect updateRect)
{
	DrawLayout(this, updateRect);
}

void ArpLView::MakeFocus(bool focusState)
{
	BView::MakeFocus(focusState);
	ArpLayoutable::SetFocusShown(focusState);
}

void ArpLView::GetPreferredSize(float* width, float* height)
{
	// Because of limitations in some Be classes, we can't report
	// a valid layout if we aren't attached to a window.
	if( !Window() ) {
		if( width ) *width = Bounds().right;
		if( height ) *height = Bounds().bottom;
		return;
	}
	
	// Retrieve layout dimensions from this tree, and return
	// the 'preferred size' constraint.
	ArpLayoutDimens dimens = LayoutDimens();
	if( width ) *width = dimens.pref_width;
	if( height ) *height = dimens.pref_height;
}

void ArpLView::ResizeToPreferred()
{
	if( Window() ) {
		ArpLayoutDimens dimens = LayoutDimens();
		ResizeLayout(dimens.pref_width, dimens.pref_height);
	}
}

void ArpLView::ComputeDimens(void)
{
	ArpD(cdb << ADH << "Computing ArpLView dimensions...\n");
	ArpLayoutable::ComputeDimens();
}
